﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using Chart_Test.Helper;

namespace Chart_Test
{
    public class DataPresenter : INotifyPropertyChanging, INotifyPropertyChanged
    {

        private List<DataSeries> _DataSeries = null;
        public List<DataSeries> DataSeries
        {
            get
            {
                return _DataSeries;
            }
            set
            {
                if (value != _DataSeries)
                {
                    _DataSeries = value;
                    OnPropertyChanged("DataSeries");
                }
            }
        }

        internal void LoadData(int days, int jobCount, int pointThickness)
        {
            DateTime start = DateTime.Today;
            DateTime end = start.AddDays(days);

            Dictionary<Job, DataSeries> jobSeries = GetJobDictionary(start, end, jobCount);
            Job[] jobs = jobSeries.Keys.ToArray();
            Dictionary<Job, double> jobHours = new Dictionary<Job, double>(jobs.Length);
            foreach (Job job in jobs)
            {
                jobHours[job] = 0;
            }

            DataSeries resSeriesItem = new DataSeries()
            {
                ChartType = Infragistics.Windows.Chart.ChartType.Point,
                DataMapping = "Label = Label; Value = Value; ToolTip = Value",
                DataSource = new List<DataSeriesItem>(),
                Color = "Green",
                Thickness = pointThickness,
               
            };

            var currResDataItem = new DataSeriesItem();

            while (start <= end)
            {
                currResDataItem.Value += 32;

                foreach (var job in jobs)
                {
                    var hours = job.WorkingDays.Where(day => day.Day == start).AsQueryable();
                    if (hours != null)
                    {
                        jobHours[job] += hours.Sum(day => day.Hours);
                    }
                }

                // Ressourcenzeiten
                currResDataItem.Label = start.ToString("d");
                (resSeriesItem.DataSource as List<DataSeriesItem>).Add(currResDataItem);
                currResDataItem = new DataSeriesItem();

                // Belastungszeiten schreiben
                foreach (var kvp in jobSeries)
                {
                    (kvp.Value.DataSource as List<DataSeriesItem>).Add(new DataSeriesItem()
                    {
                        Label = start.ToString("d"),
                        Value = jobHours[kvp.Key]
                    });
                    jobHours[kvp.Key] = 0;
                }

                start = start.AddDays(1);
            }

            DataSeries = new List<DataSeries>(jobSeries.Values.Concat(new List<DataSeries>() { resSeriesItem }));
        }

        private Dictionary<Job, DataSeries> GetJobDictionary(DateTime start, DateTime ende, int jobCount)
        {
            var retDict = new Dictionary<Job, DataSeries>();
            Random random = new Random(jobCount);
            Job[] jobs = new Job[jobCount];
            int days = (ende - start).Days;
            for (int i = 0; i < jobCount; i++)
            {
                jobs[i] = new Job()
                {
                    ID = i,
                    Name = string.Format("Job {0}", i),
                    Status = random.Next(7),
                    Start = start.AddDays(random.Next(days)),
                    Deadline = start.AddDays(random.Next(4)),
                    Taskname = "Random Task"
                };
                jobs[i].WorkingDays = new List<WorkDay>()
                                    {
                                        new WorkDay()
                                        {
                                            Day = jobs[i].Start,
                                            Hours = Math.Round( random.NextDouble()*8, 2)
                                        },
                                        new WorkDay()
                                        {
                                            Day = jobs[i].Start.AddDays(1),
                                            Hours = Math.Round( random.NextDouble()*8, 2)
                                        },
                                        new WorkDay()
                                        {
                                            Day = jobs[i].Start.AddDays(2),
                                            Hours = Math.Round( random.NextDouble()*8, 2)
                                        }
                                    };
            }

            foreach (Job job in jobs)
            {
                retDict[job] = new DataSeries()
                {
                    ChartType = Infragistics.Windows.Chart.ChartType.StackedColumn,
                    DataMapping = "Label = Label; Value = Value",
                    DataSource = new List<DataSeriesItem>(),
                    Color = GetColor(job.Status),
                    Thickness = 2,
                    ToolTip = string.Format("Name: {0}\r\nTask: {1}\r\nStart: {2}\r\nStatus: {3}\r\nDeadline: {4}", job.Name, job.Taskname, job.Start, job.Status, job.Deadline)
                };
            }

            return retDict;
        }

        private string GetColor(int jobStatus)
        {
            switch (jobStatus)
            {
                case 0:
                    return System.Windows.Media.Brushes.DarkViolet.ToString();
                    break;
                case 1:
                    return System.Windows.Media.Brushes.CornflowerBlue.ToString();
                    break;
                case 2:
                    return System.Windows.Media.Brushes.Yellow.ToString();
                    break;
                case 3:
                    return System.Windows.Media.Brushes.LightGreen.ToString();
                    break;
                case 4:
                    return System.Windows.Media.Brushes.LightGray.ToString();
                    break;
                case 5:
                    return System.Windows.Media.Brushes.DarkGray.ToString();
                    break;
                case 6:
                default:
                    return System.Windows.Media.Brushes.Red.ToString();
                    break;
            }
        }

        #region INotifyPropertyChanging Member

        public event PropertyChangingEventHandler PropertyChanging;

        private void OnPropertyChanging(string prop)
        {
            if (PropertyChanging != null)
            {
                PropertyChanging(this, new PropertyChangingEventArgs(prop));
            }
        }

        #endregion

        #region INotifyPropertyChanged Member

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string prop)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(prop));
            }
        }

        #endregion
    }

    public class Job
    {
        public int ID { get; set; }
        public string Name { get; set; }
        public string Taskname { get; set; }
        public DateTime Start { get; set; }
        public DateTime Deadline { get; set; }
        public int Status { get; set; }
        public List<WorkDay> WorkingDays { get; set; }
    }

    public class WorkDay
    {
        public DateTime Day { get; set; }
        public double Hours { get; set; }
    }
}
